# PyTDL

A Python package to parse ETSI TDL concrete syntax and manipulate the TDL models.

## Implementation

PyTDL uses the [textX](http://textx.github.io/textX/stable/) python package to parse and manipulate TDL definitions (in TDLan syntax) into python objects.

The grammars are developed on the basis of the Xtext grammars used in [the TOP Eclipse Plugins](https://labs.etsi.org/rep/top/ide).

The syntaxes of the grammars between `Xtext` and `textX` are different. The differences are documented [here](http://textx.github.io/textX/stable/about/comparison/#difference-to-xtext-grammar-language).

More information on ETSI TDL are available at the [TDL website](https://tdl.etsi.org/).

## Install dependencies

Install requirements via `pip`:

    python3 -m pip install textx argparse 

## Usage

Use the `pytdl` script as an executable entry point to run the main script (`pytdl.py`).

The command line arguments are exaplained below:

    usage: pytdl.py [-h] {validate,python,inspect} filename
    
    TDL python utilities.
    
    positional arguments:
      {validate,python,inspect}
                        the command to be executed.
                        "inspect"    returns a list of definitions in the given package
                        "validate"   parses the file and print any syntactic errors found. Exit code is 0 if the file is valid
                        "python"     generates python objects for the definitions found (experimental)
      filename              the TDL script to manipulate

    optional arguments:
      -h, --help            show this help message and exit
	
## Test

A very small set of tests is included in the `test/` folder. 

To execute them, first install the `pytest` python package (you may need super user rights):

    python3 -m pip pytest

Then execute the following:

    cd test/
    python3 -m pytest .

## Todo

* [x]  Import new data types from Xtext plugins 
* [x]  Verify the new grammar parses Example 1 and Example 2 of 1.5
* [ ]  Support for inline parameterBinding (required for Example 3 of 1.5)
* [ ]  Support previous examples (when possible)

## Licensing

TDL examples taken from ETSI specifications are under Copyright of ETSI ([www.etsi.org](www.etsi.org)) with
no rights granted.

All other files are released under the Eclipse Public License v1.
The complete text of the EPL is available in the LICENSE.txt file.