#!/env/python3
'''
PyTDL - validate ETSI TDL, with Python

Released under Eclipse Public License v1
'''

import argparse
import os
import sys

from textx.metamodel import metamodel_from_file
from textx.exceptions import TextXSemanticError, TextXSyntaxError
import pytdl_lib as lib

DIR = os.path.dirname(os.path.abspath(__file__))

try:
    TDL_MM = metamodel_from_file(DIR+'/../tdl_grammars/TDLan2.tx',
        classes=[
            lib.Package, 
            lib.TestDescription, 
            lib.BehaviourDescription,
            lib.CompoundBehaviour,
            lib.Block,
            lib.Message]
    )
except TextXSemanticError as err:
    print("Error in TDL grammar files:")
    print(err)
    print("Quitting.")
    exit(-1)

SUPPORTED_COMMANDS=[
    "validate",
    "python",
    "inspect"
]

CMD_HELP='''the command to be executed.
"inspect"    returns a list of definitions in the given package
"validate"   parses the file and print any syntactic errors found. Exit code is 0 if the file is valid
"python"     generates python objects for the definitions found (experimental)'''

PARSER = argparse.ArgumentParser(description='TDL python utilities.', formatter_class=argparse.RawTextHelpFormatter)
PARSER.add_argument('command', nargs=1, choices=SUPPORTED_COMMANDS, help=CMD_HELP)
PARSER.add_argument('filename', nargs=1, help='the TDL script to manipulate')

ARGS = PARSER.parse_args()

FN = ARGS.filename[0]
CMD = ARGS.command[0]

P = None
try:
    P = TDL_MM.model_from_file(FN)
except TextXSyntaxError as err:
    print("Syntax error found in file " + FN + ":")
    print("")
    print("\t" + str(err))
    print("")
    print("Quitting.")
    sys.exit(-2)

if CMD == "validate":
    print("File '{}' is valid.".format(FN))
    sys.exit(0)

if CMD == "python":
    print("import pytdl.interpreter")
    print("")

    P.to_python()
    sys.exit(0)

if CMD == "inspect":
    print("###################")
    print("Tests in the package:")

    lib.list_test_descriptions(P)

    print("###################")
    print("Everything in the package:")

    for i in P.packagedElement:
        print(i.name)

    sys.exit(0)
