#!/env/python3
'''
Library for Pytdl

Released under Eclipse Public License v1
'''

def indent(x):
    return "  " * x

class Element(object):
    """docstring for Element"""
    def __init__(self, name=None, parent=None, comment=None, annotation=None):
        super(Element, self).__init__()
        self.name = name
        self.parent = parent
        self.comment = comment
        self.annotation = annotation
        

class Package(Element):
    """docstring for TdlPackage"""
    def __init__(self, name, packagedElement, imported, nestedPackage=None, annotation=None, comment=None, parent=None):
        super(Package, self).__init__(name=name, parent=parent, comment=comment, annotation=annotation)
        self.imported = imported
        self.nestedPackage = nestedPackage
        self.packagedElement = packagedElement

    def __str__(self):
        return ("Package " + self.name + " containing: ")

    def all(self, tdl_class=None):
        '''
        Generator of packagedElements
        @tdl_class Filter class of elements to be yield
        '''
        for pel in self.packagedElement:
            if (not tdl_class) or pel.__class__.__name__ == tdl_class:
                yield pel

    def test_descriptions(self):
        return self.all("TestDescription")

    def test_configurations(self):
        return self.all("TestConfiguration")

    def to_python(self, ind=0):
        print("class Package_"+self.name+"(TDL_Package):")
        print("    self.name = "+self.name)
        for i in self.all():
            if hasattr(i, 'to_python'):
                i.to_python(ind+1)
            else:
                print(i)
        print("")

class TestDescription(Element):
    """docstring for TestDescription"""
    def __init__(self, name, behaviourDescription,
                 parent, formalParameter, testConfiguration,
                 testObjective, isLocallyOrdered, comment, annotation):
        super(TestDescription, self).__init__(name=name, parent=parent, comment=comment, annotation=annotation)
        self.behaviourDescription = behaviourDescription
        self.formalParameter = formalParameter 
        self.testConfiguration = testConfiguration
        self.testObjective = testObjective 
        self.isLocallyOrdered = isLocallyOrdered 

    def to_python(self, ind=0):
        print(indent(ind) + "def test_"+self.name+"():")
        self.behaviourDescription.to_python(ind+1)

class BehaviourDescription(Element):
    """docstring for BehaviourDescription"""
    def __init__(self, name, behaviour,
                 parent,comment, annotation):
        super(BehaviourDescription, self).__init__(name=name, parent=parent, comment=comment, annotation=annotation)
        self.behaviour = behaviour 

    def to_python(self, ind=0):
        self.behaviour.to_python(ind)

class CompoundBehaviour(Element):
    """docstring for TestDescription"""
    def __init__(self, 
                block, 
                 #comment, annotation, 
                 #testObjective, periodic, exceptional,
                parent, 
                #behaviour
                 ):
        super(CompoundBehaviour, self).__init__(name=None, parent=parent, comment=None, annotation=None)
        self.block = block
        #self.comment = comment
        #self.annotation = annotation
        #self.testObjective = testObjective
        #elf.periodic = periodic
        #elf.exceptional = exceptional
        #self.parent = parent
        #self.behaviour = behaviour

    def to_python(self, ind=0):
        # print indent(ind) + "compound_behaviour_thing()"
        self.block.to_python(ind+1)

class Block(Element):
    def __init__(self, comment, name, parent,  behaviour, guard, annotation):
        super(Block, self).__init__(name=name, parent=parent, comment=comment, annotation=annotation)
        self.comment = comment
        self.guard = guard
        self.annotation = annotation
        self.name = name
        self.behaviour = behaviour

    def to_python(self, ind):
        # print indent(ind) + "block()"
        for i in self.behaviour:
            i.to_python(ind+1)

class Message(Element):
    """docstring for Message"""
    def __init__(self, name, parent, comment, annotation, testObjective, timeLabel, target, timeConstraint, sourceGate, argument, isTrigger ):
        super(Message, self).__init__(name=name, parent=parent, comment=comment, annotation=annotation)
        self.isTrigger = isTrigger
        self.argument = argument
        self.target = target
        self.timeLabel = timeLabel
        self.timeConstraint = timeConstraint
        self.sourceGate =  sourceGate

    def to_python(self, ind):
        print(indent(ind) + "tdl_send(from="+self.sourceGate+",to="+str(self.target[0].targetGate)+")")
            

def list_test_descriptions(package):
    for tds in package.test_descriptions():
            print(tds.name)

def list_all(package):
    for i in package.all():
        print(i)
